# TOOL FOR PREPROCESSING CORPS TEXT FILES (e.g. REMOVAL OF TAGS)
# by Seong-Eun Cho
# University of Bergen
# MA Thesis (2013)

use warnings;
use strict;
use utf8;

# Grep File Names in the input folder
opendir (DIR, "./input") or die "$!";
my @files = grep {/.\.txt/}  readdir DIR;
close DIR;

# Readin Files in Specified Folder
foreach my $file (@files) {
	open(my $corpus,"./input/$file") or die "$!";
	my @input = <$corpus>;
	my $inputstring = join("",@input);
	close($corpus);
	
	# Remove Meta Data at the Beginning ({title}..{description})
	$inputstring =~ s/{title}(\s|\S)+{speech}\s+//g;
	
	# Remove Meta Data at the End ({/speech})
	$inputstring =~ s/{\/speech}(\s+)?$//g;
	
	# Remove Meta Data located inbetween (-- {APPLAUSE} --)
	$inputstring =~ s/(-- )?{.*?}( --)?//g;
	
	# Remove Meta Data located inbetween (VOICES)
	$inputstring =~ s/VOICES : //g;
	
	# Remove Meta Data located inbetween (Pauses - new sentence border)
	$inputstring =~ s/(\s)?--(\s)?([A-Z])/\. $3/g;
	
	# Remove Meta Data located inbetween (Pauses - sentences continues)
	$inputstring =~ s/(\s)?--(\s)?([^A-Z])/ $3/g;
	
	# Remove Meta Data located inbetween (DOTS)
	$inputstring =~ s/ . . . /, /g;
	
	# Remove Multiple White Spaces
	$inputstring =~ s/(\s){2,}/$1/g;
	
	# Add Punctuation at Newlines
	$inputstring =~ s/([a-z])(\n)/$1\.$2/g;
	
	# Print Output
	open(my $out, '>', "./result_plaintext/plain_$file")
		or die "Could not create output for $file\n";
		
		print $out $inputstring;
		print $file."\n";
}

