#!/usr/bin/perl
use warnings;
use strict;
use utf8;

##################################################################
													# :: READ-IN
# Load StanfordPaser Output
open(my $input, "../#5.text.tree.tagged")
	or die "Could not open file: #text.tree.tagged";
	
	# Divide in sentences
	my @input = <$input>;
	close $input;

	my $inputstring = join("",@input);
	my @sentences = split(m/\n\n/,$inputstring);

# Parallel Lists: Syntax Trees and Dependency Structure
my @trees;
my @dependencies;

	# Feed @trees with Syntax Trees (odd)
	my $i = 1; # 0=odd, 1=even
	foreach(grep $i++ % 2, @sentences)
	{
		push(@trees,$_);
	}

		# Feed @dependencies with Dependency Structures (even)
		my $j = 0; # 0=odd, 1=even
		foreach(grep $j++ % 2, @sentences)
		{
			push(@dependencies,$_);
		}

##################################################################
									# :: PREPOSITION LEXICA
									# :: VERB LEXICA (NON-MOTION)
									# :: PRONOUN LEXICA
									# :: ADVERB LEXICA

# Components of Moving Time and Moving Ego Constructions 
my $prep_MomentSense = "to|at|towards?|from|into";
my $prep_DurationSense = "through";
my $adverbs = "closer?|neare?r?|by";
my $get = "get|got|gotten|getting|gets";

# Components of Time Orientation Construction
my $prep_Sight = "on|at|to";
my $triggers_TO = "ahead|behind|in_front_of|ahead_of";
my $obj_pronouns = "me|you|him|her|it|us|them";
my $be = "are|is|was|were|been";
my $lie = "lie|lay|lain|lying";


##################################################################
									# :: Variables for Statistics
my %stats_MT;
	$stats_MT{"MOTION VERB + TIME/EVENT"} = "0";
	$stats_MT{"REDUCED RELATIVE CLAUSE"} = "0";
	$stats_MT{"MOTION: ADJECTIVAL PARTICIPLE"} = "0";
	$stats_MT{"GO BY + TIME/EVENT"} = "0";

my %stats_ME;
	$stats_ME{"MOTION VERB + TIME (Moment Sense)"} = "0";
	$stats_ME{"NON-MOTION VERB + TIME (Moment Sense)"} = "0";
	$stats_ME{"THROUGH/TROUGHOUT (Duration Sense)"} = "0";

my %stats_TO;
	$stats_TO{"SIGHT VERB + TIME/EVENT"} = "0";
	$stats_TO{"AHEAD/BEHIND + PERS. PRONOUN + TIME/EVENT"} = "0";
	$stats_TO{"AHEAD/BEHIND + TIME/EVENT"} = "0";
	$stats_TO{"LIE + AHEAD/BEHIND + TIME/EVENT"} = "0";



##################################################################
													# :: ANALYSIS
##################################################################

# Counter for going through all Dependencies
my $counter = "0";

# Analyze each Dependency Structure for each Sentence separately
foreach my $dependency (@dependencies) {

		#############################################
					# MOVING TIME ANALYSIS
		#############################################
			
		# A) SEARCH FOR INSTANCES OF [MOTION VERB] AND [TIME|EVENT] DEPENDENCIES
		# ---------------------------------------------------------------------
				# - And it is a proposal whose [time: TIME] has [come]
				# - The [time: TIME] for bickering has [passed]. 
				# - But each [day: TIME] [went] by without any orders 
				# - In the [days: TIME] that [followed]
				if ($dependency =~ m/nsubj\((motion_mE?T:(.+?)-\d+), (time|event)-\d+\)/) {
						
					# Temp Variables; needed for next step
					my $nsubjMotion = $1;
					my $motionVerb = $2;
						
							# Ignore sentence if it contains another (non-temporal) subject / a direct object
							if ($dependency =~ m/((dobj|nsubj|prep_.+?)\($nsubjMotion, ((?!(time|event))))/) {
								0;
							}
					
							# RETURN SENTENCE
							else {
								print &ReturnSentence($trees[$counter],$motionVerb)." - MOVING TIME\n\n";
								$stats_MT{"MOTION VERB + TIME/EVENT"}++;
							}
				}
			
			
		
			# B) REDUCED RELATIVE CLAUSES
					# - not just the next [generation: TIME] but many [generations: TIME] to [come]
					# - together in the [months: TIME] and [years: TIME] to [come] 
					elsif ($dependency =~ m/appos\((time|event)-\d+, .+?-(\d+)\)\naux\((motion_mE?T:(.+?)-\d+), to-(\d+)\)/) {
									
								# RETURN SENTENCE
								print &ReturnSentence($trees[$counter],$3)." - MOVING TIME_RRC\n\n";
								$stats_MT{"REDUCED RELATIVE CLAUSE"}++;
					}
			

		
			# C) [MOTION: ADJECTIVAL PARTICIPLE] + [TIME|EVENT]
			# ---------------------------------------------------------------------
					# - each and every [day: TIME] in the [coming] [months: TIME].
					# - And that we're going to see this [year: TIME] [coming] up.
			
					if ($dependency =~ m/(amod|partmod)\((time|event)-\d+, motion_mE?T:(.+?)-\d+\)/) {
				
						# RETURN SENTENCE
						print &ReturnSentence($trees[$counter],$3)." - MOVING TIME_ADJ\n\n";
						$stats_MT{"MOTION: ADJECTIVAL PARTICIPLE"}++;
					}
				
			# D) SEARCH FOR INSTANCES OF [GO BY] CONSTRUCTIONS 
			# ---------------------------------------------------------------------
					# - But each [day: TIME] [went by] without any orders
					# - cost-cutting measures as the [years: TIME] [go by]
						if ($dependency =~ m/nsubj\((motion_mE:(go|went|gone|going|goes))-\d+, (time|event)-\d+\)(\s|\S)+(advmod|acomp|prep)\((motion_mE:(go|went|gone|going|goes))-\d+, ($adverbs)-\d+\)/) {
					
					# RETURN SENTENCE
					print &ReturnSentence($trees[$counter],$6." ".$8)." - MOVING TIME_GOBY\n\n";
					$stats_MT{"GO BY + TIME/EVENT"}++;
					}
				
							
		#############################################
						# Moving Ego
		#############################################
		# Search Line by Line
		my @depUnit = split("\r", $dependency);
		
		# GO THROUGH EACH DEPENDENCY
		foreach (@depUnit) {
				
				# A) SEARCH FOR INSTANCES OF [MOTION VERB] AND [TIME] DEPENDENCIES
				# ---------------------------------------------------------------------
					# - we will have [moved] closer to that [day: TIME]
					# - we're going to [go] through this [year: TIME] 
					# - We're not going to [go] back to those [days: TIME]
					# - as we [head] into the [spring: TIME] and [summer: TIME] [months: TIME]
				
					# Only search for time
					if ($_ =~ m/(prep_($prep_MomentSense)|dobj)\(motion_mET?:(.+?)-(\d+), (time)-(\d+)\)/ig) {
							
						my $motionVerb = $3;
						my $verbIndex = $4;
						my $timeIndex = $6;
							
							if ($motionVerb eq "going") {
								
									# Ignore sentence if the dependency list contains a "going to [infinitive]"-construction
									if ($dependency =~ m/aux\((.+?-\d+), to-\d+\)\s(.+?\s)?xcomp\(motion_mET?:going-$verbIndex, .+?-\d+\)/ig) {
										0;
									}		
								
									# Ignore sentence if it contains another (non-temporal) object
									elsif ($dependency =~ m/((prep_.+?|dobj)\(motion_mET?:$motionVerb-$verbIndex, (?!time))/g) {
										0;
									}
								
									else {
										
										# RETURN SENTENCE
										print &ReturnSentence($trees[$counter],$motionVerb)." - MOVING EGO\n\n";
										$stats_ME{"MOTION VERB + TIME (Moment Sense)"}++;
									}
							}
						
							else {
								
									# Ignore sentence if it contains another (non-temporal) object
									if ($dependency =~ m/((prep_.+?|dobj)\(motion_mET?:$motionVerb-$verbIndex, (?!time))/ig) {
										0;
									}
				
									else {
										
										# RETURN SENTENCE
										print &ReturnSentence($trees[$counter],$motionVerb)." - MOVING EGO\n\n";
										$stats_ME{"MOTION VERB + TIME (Moment Sense)"}++;
									}
							}
					}
				
				
				# B) SEARCH FOR INSTANCES OF [NON-MOTION VERB: get] AND [TIME] DEPENDENCIES
				# ---------------------------------------------------------------------
					# - [get] through some very difficult [times: TIME]
					# - Now, some may have thought it would take [months: TIME] to [get] to this [point: TIME]
					
					if ($dependency =~ m/prep_to\(($get)-\d+, time-\d+\)/) {
						my $trigger = $1;
						
						# RETURN SENTENCE
						print &ReturnSentence($trees[$counter],$trigger)." - MOVING EGO_GETI\n\n";
						$stats_ME{"NON-MOTION VERB + TIME (Moment Sense)"}++;
					}
					
				# C) SEARCH FOR INSTANCES OF THE DURATION SENSE (through)	
					# - America we are passing [through] a [time: TIME] of great trial.
					# - [Throughout] our [past: TIME], there have been [times: TIME] when markets have fallen out of balance.
					
					if ($dependency =~ m/prep_($prep_DurationSense)\((.+?)-\d+, time-\d+\)/ig) {
						print &ReturnSentence($trees[$counter],$1)." - MOVING EGO_DURA\n\n";
						$stats_ME{"THROUGH/TROUGHOUT (Duration Sense)"}++;
					}
				}
					
		
									 				
					
					############################################
							#	Temporal Sequence
					############################################
					
					# A) Search for instances of [SIGHT Verb] and [Time/Event] dependencies
					# ---------------------------------------------------------------------
						# - She expects to [face] more than a [month: TIME] of aggressive chemotherapy.
						# - we were [looking] at the possible [end: TIME] of General Motors.
						# - we're going to be able to [look] back at the [end: TIME] of this [year: TIME]
						
						if ($dependency =~ m/(prep_($prep_Sight)|dobj)\(sight_TO:(.+?)-(\d+), (time)-(\d+)\)/ig) {
					
							# Temp Variables; needed for next step
							my $sightVerb = $3;
							my $sightIndex = $4;
						
							# Ignore sentence if it contains another a direct object
									if ($dependency =~ m/(dobj|prep_.+?)\(sight_TO:(.+?)-(\d+), ((?!time))/) {
										my $sightVerb2 = $2;
										my $sightIndex2 = $3;
											
											# Check if Verbs are Co-indexed
											if (($sightVerb eq $sightVerb2) && ($sightIndex == $sightIndex2)) {
												0;
											}
									
											# RETURN SENTENCE
											else {
												print &ReturnSentence($trees[$counter],$sightVerb)." - TIME ORIENTATION\n\n";
												$stats_TO{"SIGHT VERB + TIME/EVENT"}++;
											}
									}
							
									# RETURN SENTENCE		
									else {
										print &ReturnSentence($trees[$counter],$sightVerb)." - TIME ORIENTATION\n\n";
										$stats_TO{"SIGHT VERB + TIME/EVENT"}++;
									}
						}
					
					
					# B) AHEAD/BEHIND RULES
					# ---------------------------------------------------------------------
					# B.1) ahead/behind and personal pronoun
						# - We hope that the worst is [behind us] 
						# - confident that our best [days: TIME] are still [ahead of us]. 		
						
						if ($dependency =~ m/prep_($triggers_TO)\(($be)-\d+, ($obj_pronouns)-\d+\)/ig) {
							my $tempseq = $1." ".$3;
							
								# RETURN SENTENCE
								print &ReturnSentence($trees[$counter],$tempseq)." - TIME ORIENTATION_AHBE\n\n";
								$stats_TO{"AHEAD/BEHIND + PERS. PRONOUN + TIME/EVENT"}++;
						}	
					
					# B.2) ahead/behind without personal pronoun
						# - There is a great [day: TIME] [ahead].
						# - brighter [days: TIME] are still [ahead]
						# - We've still got some tough [times: TIME] [ahead]
						
						if ($dependency =~ m/nsubj\(($be)-(\d+), (time|event)-\d+\)(\s|\S)+advmod\(($be)-(\d+), ($triggers_TO)-\d+\)/ig){
									my $trigger = $7;
										if ($2 == $6) {
											
											# RETURN SENTENCE
											print &ReturnSentence($trees[$counter],$trigger)." - TIME ORIENTATION_AHBE\n\n";
											$stats_TO{"AHEAD/BEHIND + TIME/EVENT"}++;
										}
						}		
						
						elsif ($dependency =~ m/npadvmod\(($triggers_TO)-\d+, (time|event)-\d+\)/ig){
							my $trigger = $1;
									
									# RETURN SENTENCE
									print &ReturnSentence($trees[$counter],$trigger)." - TIME ORIENTATION_AHBE\n\n";
									$stats_TO{"AHEAD/BEHIND + TIME/EVENT"}++;
						}
						elsif ($dependency =~ m/appos\((time|event)-\d+, .+?-(\d+)\)\sadvmod\(.+?-\d+, ($triggers_TO)-(\d+)\)/ig){
							my $trigger = $3;			
										if ($4-$2==2) {
											
											# RETURN SENTENCE
											print &ReturnSentence($trees[$counter],$trigger)." - TIME ORIENTATION_AHBE\n\n";
											$stats_TO{"AHEAD/BEHIND + TIME/EVENT"}++;
										}
						}
					
					# B.3) lie + ahead/behind
						# - Great [days: TIME] [lie ahead] for this nation.
						# - And I am confident that better [days: TIME] [lie ahead]
						if ($dependency=~ m/nsubj\(($lie)-(\d+), (event|time)-\d+\)(\s|\S)+advmod\(($lie)-(\d+), ($triggers_TO)-(\d+)\)/ig) {
							my $trigger = $5." ".$7;
							if ($2 == $6) {
								
								# RETURN SENTENCE
								print &ReturnSentence($trees[$counter],$trigger)." - TIME ORIENTATION_LIE\n\n";
								$stats_TO{"LIE + AHEAD/BEHIND + TIME/EVENT"}++;
							}
						}
		
		# Go To Next Dependency					
		$counter++;
		
}

# SUBROUTINE: RETURN SENTENCE + TAG TRIGGERS
sub ReturnSentence {
	my $tree = $_[0];
	my $trigger = $_[1];
	$tree =~ s/\([A-Z\$]+\s+//g;
	$tree =~ s/\(\-LRB\- \-LRB\-\)\s+/\n\[/g;
	$tree =~ s/\s+\(\-RRB\- \-RRB\-\)/\]\n/g;
	#17.03. \d added
	$tree =~ s/([^A-Za-z\d\s]){2,}/$1/g;
	$tree =~ s/[()]//g;
	$tree =~ s/\s+/ /g;
	#17.03. \d added
	$tree =~ s/ ([^A-Za-z\d[])/$1/g;
	$tree =~ s/ n't/n't/g;
	$tree =~ s/(time) \[(.+?)\]/\[$2: TIME\]/g;
	$tree =~ s/(event) \[(.+?)\]/\[$2: EVENT\]/g;
	
	# Phrasal Verb
	$trigger =~ s/\S+\((\S+)-\d+, (\S+)-\d+\)/$1 $2/g;
	
	# come-15 -> come
	$trigger =~ s/(.+)?-\d+/$1/g;
	$trigger =~ s/motion_mE?T?://g;
	
	# prepc_by(going-30
	$trigger =~ s/prepc_(by)\((\S+)-\d+/$2 $1/g;

	# ahead_of/in_front_of
	$trigger =~ s/_/ /g;
	
	# Tag (phrasal) Verb	
	$tree =~ s/(\b)($trigger)(\b)/$1\[$2\]$3/ig;
	return $tree;
}

# STATISTICS:

print "\n\nTIME ORIENTATION\n";
print "\tSIGHT VERB + TIME/EVENT:\t\t".$stats_TO{"SIGHT VERB + TIME/EVENT"}."\n";
print "\tAHEAD/BEHIND + PERS.PRON + TIME/EVENT:\t".$stats_TO{"AHEAD/BEHIND + PERS. PRONOUN + TIME/EVENT"}."\n";
print "\tAHEAD/BEHIND + TIME/EVENT:\t\t".$stats_TO{"AHEAD/BEHIND + TIME/EVENT"}."\n";
print "\tLIE + AHEAD/BEHIND + TIME/EVENT:\t".$stats_TO{"LIE + AHEAD/BEHIND + TIME/EVENT"}."\n";
print "\t\t\t\t\tTOTAL:\t";
print $stats_TO{"SIGHT VERB + TIME/EVENT"}+$stats_TO{"AHEAD/BEHIND + PERS. PRONOUN + TIME/EVENT"}+$stats_TO{"AHEAD/BEHIND + TIME/EVENT"}+$stats_TO{"LIE + AHEAD/BEHIND + TIME/EVENT"}."\n\n";

print "MOVING TIME\n";
print "\tMOTION VERB + TIME/EVENT:\t\t".$stats_MT{"MOTION VERB + TIME/EVENT"}."\n";
print "\tREDUCED RELATIVE CLAUSE:\t\t".$stats_MT{"REDUCED RELATIVE CLAUSE"}."\n";
print "\tADJECTIVAL PARTICIPLE:\t\t\t".$stats_MT{"MOTION: ADJECTIVAL PARTICIPLE"}."\n";
print "\tGET/WENT BY + TIME/EVENT:\t\t".$stats_MT{"GO BY + TIME/EVENT"}."\n";
print "\t\t\t\t\tTOTAL:\t";
print $stats_MT{"MOTION VERB + TIME/EVENT"}+$stats_MT{"REDUCED RELATIVE CLAUSE"}+$stats_MT{"MOTION: ADJECTIVAL PARTICIPLE"}+$stats_MT{"GO BY + TIME/EVENT"}."\n\n";

print "MOVING EGO\n";
print "\tMOTION VERB + TIME (Mom. Sense ):\t".$stats_ME{"MOTION VERB + TIME (Moment Sense)"}."\n";
print "\tNON-MOTION VERB + TIME (Mom. Sense):\t".$stats_ME{"NON-MOTION VERB + TIME (Moment Sense)"}."\n";
print "\tTHROUGH/TROUGHOUT + TIME (Dur. Sense):\t".$stats_ME{"THROUGH/TROUGHOUT (Duration Sense)"}."\n";
print "\t\t\t\t\tTOTAL:\t";
print $stats_ME{"MOTION VERB + TIME (Moment Sense)"}+$stats_ME{"NON-MOTION VERB + TIME (Moment Sense)"}+$stats_ME{"THROUGH/TROUGHOUT (Duration Sense)"}."\n\n";





# CODE GRAVEYARD:

# OLD VERSION:
# if ($dependency =~ m/nsubj\(motion_MT:(.+?)-\d+, (time|event)-\d+\)/ig) {
#   	print &ReturnSentence($trees[$counter],$1)." - MOVING TIME\n\n";
# }

# # # MOVING TIME (Phrasal Verbs)
# # # Examples: draw/get closer/near
# if ($dependency =~ m/nsubj\(($get|$draw)-\d+, (time|event)-\d+\)(\s|\S)+((advmod|acomp|prep)\(($get|$draw)-\d+, ($adverbs)-\d+\))/ig) {	
# 	print &ReturnSentence($trees[$counter],$4)." - MOVING TIME [Phrasal Verbs]\n\n";
# }

# TEST: 
#print "\n\t[ ".&ReturnSentence($trees[$counter],$motionVerb)." ]\n\n";

# foreach (@depUnit) {
# 	if ($_ =~ m/(prep_.+?|dobj)\(motion_mET?:$motionVerb-$verbIndex, (.+?)-(\d+)/) {
# 		if ($timeIndex <= $3) {
# 			print &ReturnSentence($trees[$counter],$motionVerb)." - MOVING EGO\n\n"
# 		}
# 		else {
# 			# TEST: 
# 			# print "\n\t[ ".&ReturnSentence($trees[$counter],$motionVerb)." ]\n\n";
# 			
# 		}
# 	}
# }

# foreach (@depUnit) {
# 	if ($_ =~ m/(prep_.+?|dobj)\(motion_mET?:$motionVerb-$verbIndex, (.+?)-(\d+)/) {
# 		if ($timeIndex <= $3) {
# 			print &ReturnSentence($trees[$counter],$motionVerb)." - MOVING EGO\n\n"
# 		}
# 		else {
# 			# TEST: 
# 			# print "\n\t[ ".&ReturnSentence($trees[$counter],$motionVerb)." ]\n\n";
# 			
# 		}
# 	}
# }


		# OLD VERSION
		# if ($dependency =~ m/(prep_($prepositions)|dobj)\(motion_mET?:(.+?)-\d+, (time|event)-\d+\)/ig) {
		# 	print &ReturnSentence($trees[$counter],$3)." - MOVING EGO\n\n";
		# }

		
			# OLD VERSION		
			# if ($dependency =~ m/nsubj\(($verb_tempseq)-\d+, (time|event)-\d+\)(\s|\S)+dobj\(($verb_tempseq)-\d+, (time|event)-\d+\)/ig) {
			# 	print &ReturnSentence($trees[$counter],$1)." - TEMPORAL SEQUENCE\n\n";
			# }
			
			# # Example: Bad times are behind us now.
			# if ($dependency =~ m/nsubj\(($be)-\d+, (time|event)-\d+\)(\s|\S)+prep_($prep_TempSeq)\(($be)-\d+, ($obj_pronouns)-\d+\)/ig) {
			# 	my $tempseq = $4." ".$6;
			# 	print &ReturnSentence($trees[$counter],$tempseq)." - TEMPORAL SEQUENCE1\n\n";
			# }


#--------------------------------


			# #############################################
			# 				# Moving Ego
			# #############################################
			# # Search Line by Line
			# my @depUnit = split("\r", $dependency);
			# 
			# # GO THROUGH EACH DEPENDENCY
			# foreach (@depUnit) {
			# 		
			# 		# A) SEARCH FOR INSTANCES OF [MOTION VERB] AND [TIME] DEPENDENCIES
			# 		# ---------------------------------------------------------------------
			# 			# - we will have [moved] closer to that [day: TIME]
			# 			# - we're going to [go] through this [year: TIME] 
			# 			# - We're not going to [go] back to those [days: TIME]
			# 			# - as we [head] into the [spring: TIME] and [summer: TIME] [months: TIME]
			# 		
			# 			# Only search for time
			# 			if ($_ =~ m/(prep_($prepositions)|dobj)\(motion_mET?:(.+?)-(\d+), (time)-(\d+)\)/ig) {
			# 					
			# 				my $motionVerb = $3;
			# 				my $verbIndex = $4;
			# 				my $timeIndex = $6;
			# 					
			# 					if ($motionVerb eq "going") {
			# 						
			# 							# Ignore sentence if the dependency list contains a "going to [infinitive]"-construction
			# 							if ($dependency =~ m/aux\((.+?-\d+), to-\d+\)\s(.+?\s)?xcomp\(motion_mET?:going-$verbIndex, .+?-\d+\)/ig) {
			# 								0;
			# 							}		
			# 						
			# 							# Ignore sentence if it contains another (non-temporal) object
			# 							elsif ($dependency =~ m/((prep_.+?|dobj)\(motion_mET?:$motionVerb-$verbIndex, (?!time))/g) {
			# 								0;
			# 							}
			# 						
			# 							else {
			# 								
			# 								# RETURN SENTENCE
			# 								print &ReturnSentence($trees[$counter],$motionVerb)." - MOVING EGO\n\n";
			# 								$stats_ME{"MOTION VERB + TIME (Moment Sense)"}++;
			# 							}
			# 					}
			# 				
			# 					else {
			# 						
			# 							# Ignore sentence if it contains another (non-temporal) object
			# 							if ($dependency =~ m/((prep_.+?|dobj)\(motion_mET?:$motionVerb-$verbIndex, (?!time))/g) {
			# 								0;
			# 							}
			# 		
			# 							else {
			# 								
			# 								# RETURN SENTENCE
			# 								print &ReturnSentence($trees[$counter],$motionVerb)." - MOVING EGO\n\n";
			# 								$stats_ME{"MOTION VERB + TIME (Moment Sense)"}++;
			# 							}
			# 					}
			# 			}
			# 		
			# 		
			# 			# B) SEARCH FOR INSTANCES OF PHRASAL VERBS
			# 				# - We are [getting closer] to the [end: TIME] of the [match: EVENT]
			# 				# - As we [get closer] to the [end: TIME] of another [year: TIME]
			# 				if ($dependency =~ m/advmod\(($get)-\d+, ($adverbs)-\d+\)(\s|\S)+prep_.+?\(($get)-\d+, time-\d+\)/ig) {
			# 					my $verbadverb = $1." ".$2;
			# 					
			# 					# RETURN SENTENCE
			# 					print &ReturnSentence($trees[$counter],$verbadverb)." - MOVING EGO\n\n";
			# 					$stats_ME{"GO BY + TIME/EVENT"}++;
			# 				}
			# 		
			# 			# C) SEARCH FOR INSTANCES OF [NON-MOTION VERB] AND [TIME] DEPENDENCIES
			# 			# ---------------------------------------------------------------------
			# 				# - We've [been] through tough [times: TIME] before
			# 				# - [get] through some very difficult [times: TIME]
			# 				elsif ($dependency =~ m/prep_(through|to)\(($get|$be)-\d+, time-\d+\)/) {
			# 					my $trigger = $2;
			# 					
			# 					# RETURN SENTENCE
			# 					print &ReturnSentence($trees[$counter],$trigger)." - MOVING EGO\n\n";
			# 					$stats_ME{"NON-MOTION VERB + TIME (Moment Sense)"}++;
			# 				}
			# 			}


