# TOOL FOR CREATION OF EVENT LEXICA WITH WORDNET
# by Seong-Eun Cho
# University of Bergen
# MA Thesis (2013)

use warnings;
use strict;
use utf8;
use Storable;

# Load Perl Module Lingua::EN::Inflect to Create Plural Forms
BEGIN { unshift @INC, "../modules/Lingua-EN-Inflect-1.89/lib"; }

use Lingua::EN::Inflect qw ( PL PL_N PL_V PL_ADJ NO NUM
                             PL_eq PL_N_eq PL_V_eq PL_ADJ_eq
                             A AN
                             PART_PRES
                             ORD NUMWORDS
                             inflect classical
                             def_noun def_verb def_adj def_a def_an );

# Load/Save WordNet Noun Lexicon
open my $input, '<', "data.noun"
	or die "Could not open file\n";
my @lexicon = <$input>;
close $input;

# Time Nouns Lexicon
my %EventLex;
my %lexicon_entries;

# Keep Nouns only
foreach (@lexicon) {
	if ($_ =~ m/^\s/) {
		$_ = 0;
	}
	elsif ($_ =~ m/n \S{2} (.+? \d )\d{3}/) {
		my $string = $1;
		my @temp = split(/ \d /,$string);
		foreach (@temp) {
			$_ =~ s/_/ /g;
			$lexicon_entries{"$_"}++;
		}
	}
}

# Print Number of Keys
print "\nTotal Number of Noun Entries: ".scalar(keys %lexicon_entries)."\n\n";

# Set Environment Variable: WORDNET (WordNet Must be Installed!)
$ENV{'PATH'} = '/usr/local/Cellar/wordnet/3.0/bin';

# Go Through Every Lexicon Entry
foreach (keys %lexicon_entries) {	
	print $_."\n";
	
	my $wnSenses = `wn "$_" -hypen`;
	
	# If the event description is found in the first sense
	# Case 1) In total only 1 sense available
	if (($wnSenses !~ m/(Sense 2)/) && ($wnSenses =~ m/(social event)/)){

		$EventLex{$_} = "1";
		print "   '------> 1\n";
		
		$EventLex{&PL($_)} = "1";
		print &PL($_)."\n";
		print "   '------> 1\n";
	}
		
	
	# Case 2) If the event description is found in the second sense + In total 2 or more senses available 
	elsif ($wnSenses =~ m/(social event)(.|\n)+Sense 2/) {
		
		$EventLex{$_} = "1";
		print "   '------> 1\n";
		
		$EventLex{&PL($_)} = "1";
		print &PL($_)."\n";
		print "   '------> 1\n";
		
	}
	

	# If the event description is found in the second sense
	elsif ($wnSenses =~ m/Sense 2(.|\n)+(social event)(.|\n)+Sense 3/) {

		$EventLex{$_} = "2";
		print "   '------> 2\n";
		
		$EventLex{&PL($_)} = "2";
		print &PL($_)."\n";
		print "   '------> 2\n";
		
	}		

	# If the event description is found in sense three or higher	
	elsif ($wnSenses =~ m/Sense 3(.|\n)+(social event)(.|\n)+Sense 4/) {
	
		$EventLex{$_} = "3";
		print "   '------> 3\n";
				
	}
				
}

print "Creating Lexica\n\n";

# Event Lexica with Different Length
my %EventLex5;
my %EventLex4;
my %EventLex3;
my %EventLex2;
my %EventLex1;

# Calc Length of Entries
foreach my $key (keys %EventLex) {
	if ($key =~ m/\S+\s\S+\s\S+\s\S+\s\S+/) {
			$EventLex5{$key} = $EventLex{$key};
	}
	elsif ($key =~ m/\S+\s\S+\s\S+\s\S+/) {
			$EventLex4{$key} = $EventLex{$key};
	}
	elsif ($key =~ m/\S+\s\S+\s\S+/) {
			$EventLex3{$key} = $EventLex{$key};
	}
	elsif ($key =~ m/\S+\s\S+/) {
			$EventLex2{$key} = $EventLex{$key};
	}
	elsif ($key =~ m/\S+/) {
			$EventLex1{$key} = $EventLex{$key};
	}
}

# Store Dictionaries
store \%EventLex5, "EventLex5";
store \%EventLex4, "EventLex4";
store \%EventLex3, "EventLex3";
store \%EventLex2, "EventLex2";
store \%EventLex1, "EventLex1";
