# TOOL FOR CREATION OF TIME LEXICA WITH WORDNET
# by Seong-Eun Cho
# University of Bergen
# MA Thesis (2013)

use warnings;
use strict;
use utf8;
use Storable;

# Load Perl Module Lingua::EN::Inflect to Create Plural Forms
BEGIN { unshift @INC, "../modules/Lingua-EN-Inflect-1.89/lib"; }

use Lingua::EN::Inflect qw ( PL PL_N PL_V PL_ADJ NO NUM
                             PL_eq PL_N_eq PL_V_eq PL_ADJ_eq
                             A AN
                             PART_PRES
                             ORD NUMWORDS
                             inflect classical
                             def_noun def_verb def_adj def_a def_an );

# Load/Save WordNet Noun Lexicon
open my $input, '<', "data.noun"
	or die "Could not open file\n";
my @lexicon = <$input>;
close $input;

# Time Nouns Lexicon
my %TimeLex;
my %lexicon_entries;

# Keep Nouns only
foreach (@lexicon) {
	if ($_ =~ m/^\s/) {
		$_ = 0;
	}
	elsif ($_ =~ m/n \S{2} (.+? \d )\d{3}/) {
		my $string = $1;
		my @temp = split(/ \d /,$string);
		foreach (@temp) {
			$_ =~ s/_/ /g;
			$lexicon_entries{"$_"}++;
		}
	}
}

# Print Number of Keys
print "\nTotal Number of Noun Entries: ".scalar(keys %lexicon_entries)."\n\n";

# Set Environment Variable: WORDNET (WordNet Must be Installed!)
$ENV{'PATH'} = '/usr/local/Cellar/wordnet/3.0/bin';

# Go Through Every Lexicon Entry
foreach (keys %lexicon_entries) {	
	print $_."\n";
	
	my $wnSenses = `wn "$_" -hypen`;
	
	# If the time description is found in the first sense
	# Case 1) In total only 1 sense available
	if (($wnSenses !~ m/(Sense 2)/) && ($wnSenses =~ m/(time period, period of time, period|time unit, unit of time|point, point in time)/)){
		
		$TimeLex{$_} = "1";
		print "   '------> 1\n";
		
		$TimeLex{&PL($_)} = "1";
		print &PL($_)."\n";
		print "   '------> 1\n";
			
	}		

	
	# Case 2) In total 2 or more senses available 
	elsif ($wnSenses =~ m/(time period, period of time, period|time unit, unit of time|point, point in time)(.|\n)+Sense 2/) {
	
		$TimeLex{$_} = "1";
		print "   '------> 1\n";
		
		$TimeLex{&PL($_)} = "1";
		print &PL($_)."\n";
		print "   '------> 1\n";
	
	}


	# If the time description is found in the second sense
	elsif ($wnSenses =~ m/Sense 2(.|\n)+(time period, period of time, period|time unit, unit of time|point, point in time)(.|\n)+Sense 3/) {
	
		$TimeLex{$_} = "2";
		print "   '------> 2\n";	
		
		$TimeLex{&PL($_)} = "2";
		print &PL($_)."\n";
		print "   '------> 2\n";
	
	}		

	
	# If the time description is found in sense three or higher
	elsif ($wnSenses =~ m/Sense 3(.|\n)+(time period, period of time, period|time unit, unit of time|point, point in time)(.|\n)+Sense 4/) {
	
		$TimeLex{$_} = "3";
		print "   '------> 3\n";
		
		$TimeLex{&PL($_)} = "3";
		print &PL($_)."\n";
		print "   '------> 3\n";	
		
	}
	
	
}

# Time Lexica with Different Length
my %TimeLex6;
my %TimeLex5;
my %TimeLex4;
my %TimeLex3;
my %TimeLex2;
my %TimeLex1;

# Calc Length of Entries
foreach my $key (keys %TimeLex) {
	if ($key =~ m/\S+\s\S+\s\S+\s\S+\s\S+/) {
			$TimeLex5{$key} = $TimeLex{$key};
	}
	elsif ($key =~ m/\S+\s\S+\s\S+\s\S+/) {
			$TimeLex4{$key} = $TimeLex{$key};
	}
	elsif ($key =~ m/\S+\s\S+\s\S+/) {
			$TimeLex3{$key} = $TimeLex{$key};
	}
	elsif ($key =~ m/\S+\s\S+/) {
			$TimeLex2{$key} = $TimeLex{$key};
	}
	elsif ($key =~ m/\S+/) {
			$TimeLex1{$key} = $TimeLex{$key};
	}
}

# Load/Save Wiki Public Holidays List
open my $holiday_input, '<', "HolidayList"
	or die "Could not open file\n";
	
my @lexicon_holiday = <$holiday_input>;
close $input;

# Add Holidays 
my $counter = 0;

foreach (@lexicon_holiday) {
	if ($_ =~ m/\S+\s\S+\s\S+\s\S+\s\S+\s\S+/) {
			chomp($_);
			$TimeLex6{$_} = "1";
			$counter = $counter+1;
	}
	elsif ($_ =~ m/\S+\s\S+\s\S+\s\S+\s\S+/) {
			chomp($_);
			$TimeLex5{$_} = "1";
			$counter = $counter+1;
	}
	elsif ($_ =~ m/\S+\s\S+\s\S+\s\S+/) {
			chomp($_);
			$TimeLex4{$_} = "1";
			$counter = $counter+1;
	}
	elsif ($_ =~ m/\S+\s\S+\s\S+/) {
			chomp($_);
			$TimeLex3{$_} = "1";
			$counter = $counter+1;
	}
	elsif ($_ =~ m/\S+\s\S+/) {
			chomp($_);
			$TimeLex2{$_} = "1";
			$counter = $counter+1;
	}
	elsif ($_ =~ m/\S+/) {
			chomp($_);
			$TimeLex1{$_} = "1";
			$counter = $counter+1;
	}
}

print "\n\n".$counter." International Holiday Names Added!\n\n";

# Store Dictionaries
store \%TimeLex6, "TimeLex6";
store \%TimeLex5, "TimeLex5";
store \%TimeLex4, "TimeLex4";
store \%TimeLex3, "TimeLex3";
store \%TimeLex2, "TimeLex2";
store \%TimeLex1, "TimeLex1";
