# TOOL FOR CREATION OF VERB LEXICA
# by Seong-Eun Cho
# University of Bergen
# MA Thesis (2013)

use strict;
use warnings;
use Data::Dumper;
use Storable;

# Moving Ego/Time Lexicon
my %lexicon;

# Read-In List of Agentive Verbs of Motion (baseforms)
# Read-In: Moving EGO
open my $baseforminput_movingego, '<', 'base_movingEgo'
	or die "Could not open file: base-forms";
	
my @baseforms_movingego = <$baseforminput_movingego>;
chomp(@baseforms_movingego);

# Read-In: Moving TIME
open my $baseforminput_movingtime, '<', 'base_movingTime'
	or die "Could not open file: base-forms";
	
my @baseforms_movingtime = <$baseforminput_movingtime>;
chomp(@baseforms_movingtime);

# Read-In: Time Orientation
open my $baseforminput_timeorientation, '<', 'base_timeorientation'
	or die "Could not open file: base-forms";
	
my @baseforms_timeorientation = <$baseforminput_timeorientation>;
chomp(@baseforms_timeorientation);


# Verblist found in Nodebox (V.1.9.4.2) <http://nodebox.net/code/index.php/Linguistics>
# Location in Nodebox: en/verb/verb.txt
open my $verbformsinput, '<', 'verb.txt'
	or die "Could not open file: verb.txt";
	
my @verbforms = <$verbformsinput>;
chomp(@verbforms);

# Save Entries here
my %verblist_movingego;
my %verblist_movingtime;
my %verblist_timeorientation;

# Look Up Wordforms : MT
foreach my $baseform_movingtime (@baseforms_movingtime) {
	foreach my $verbform (@verbforms) {
		if ($verbform =~ m/^($baseform_movingtime)?,+(\w+)?,+(\w+)?,+(\w+)?,+(\w+)?,/) {
			$verblist_movingtime{$1}++;
			$verblist_movingtime{$2}++;
			$verblist_movingtime{$3}++;
			$verblist_movingtime{$4}++;
			$verblist_movingtime{$5}++;
		}
	}	
}

# Look Up Wordforms : ME
foreach my $baseform_movingego (@baseforms_movingego) {
	foreach my $verbform (@verbforms) {
		if ($verbform =~ m/^($baseform_movingego)?,+(\w+)?,+(\w+)?,+(\w+)?,+(\w+)?,/) {
			$verblist_movingego{$1}++;
			$verblist_movingego{$2}++;
			$verblist_movingego{$3}++;
			$verblist_movingego{$4}++;
			$verblist_movingego{$5}++;
		}
	}	
}

# Look Up Wordforms : TO
foreach my $baseform_timeorientation (@baseforms_timeorientation) {
	foreach my $verbform (@verbforms) {
		if ($verbform =~ m/^($baseform_timeorientation)?,+(\w+)?,+(\w+)?,+(\w+)?,+(\w+)?,/) {
			$verblist_timeorientation{$1}++;
			$verblist_timeorientation{$2}++;
			$verblist_timeorientation{$3}++;
			$verblist_timeorientation{$4}++;
			$verblist_timeorientation{$5}++;
		}
	}	
}


# Assign Values to Entries
foreach (keys %verblist_movingtime) {
		$lexicon{$_} = "motion_mT";
}

foreach (keys %verblist_movingego) {
	
	if (exists $lexicon{$_}) {
		$lexicon{$_} = "motion_mET";
	}
	else {
		$lexicon{$_} = "motion_mE";
	}
}

foreach (keys %verblist_timeorientation) {
	$lexicon{$_} = "sight_TO";
}

# Store in VerbLex
store \%lexicon, "VerbLex";
print Dumper %lexicon;
